=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2013-05-08 13:51:52 +0000
+++ CMakeLists.txt	2013-07-08 15:55:12 +0000
@@ -30,7 +30,7 @@
 find_library(TELEPATHY_QT5_SERVICE_LIBRARIES telepathy-qt5-service)
 find_library(OFONO_QT_LIBRARIES ofono-qt)
 
-add_executable(telepathy-ofono main.cpp protocol.cpp connection.cpp ofonotextchannel.cpp ofonocallchannel.cpp voicemailiface.cpp speakeriface.cpp)
+add_executable(telepathy-ofono main.cpp protocol.cpp connection.cpp ofonotextchannel.cpp ofonocallchannel.cpp voicemailiface.cpp speakeriface.cpp mmsdmanager.cpp mmsdservice.cpp mmsdmessage.cpp)
 qt5_use_modules(telepathy-ofono Core DBus)
 
 target_link_libraries(telepathy-ofono ${Qt5Core_LIBRARIES} ${Qt5DBus_LIBRARIES} -ltelepathy-qt5 ${TELEPATHY_QT5_SERVICE_LIBRARIES} ${OFONO_QT_LIBRARIES})

=== modified file 'connection.cpp'
--- connection.cpp	2013-06-01 01:50:34 +0000
+++ connection.cpp	2013-07-08 15:55:12 +0000
@@ -30,6 +30,9 @@
 #include "phoneutils.h"
 #include "protocol.h"
 
+#include "mmsdmessage.h"
+#include "mmsdservice.h"
+
 // miliseconds
 #define OFONO_REGISTER_RETRY_TIME 5000
 
@@ -45,7 +48,8 @@
     mOfonoNetworkRegistration(new OfonoNetworkRegistration(OfonoModem::AutomaticSelect, "")),
     mOfonoMessageWaiting(new OfonoMessageWaiting(OfonoModem::AutomaticSelect, "")),
     mHandleCount(0),
-    mRegisterTimer(new QTimer(this))
+    mRegisterTimer(new QTimer(this)),
+    mMmsdManager(new MMSDManager(this))
 {
     setSelfHandle(newHandle("<SelfHandle>"));
 
@@ -138,6 +142,126 @@
     QObject::connect(mOfonoMessageWaiting, SIGNAL(voicemailWaitingChanged(bool)), voicemailIface.data(), SLOT(setVoicemailIndicator(bool)));
 
     QObject::connect(mRegisterTimer, SIGNAL(timeout()), SLOT(onTryRegister()));
+
+    QObject::connect(mMmsdManager, SIGNAL(serviceAdded(const QString&)), SLOT(onMMSDServiceAdded(const QString&)));
+    QObject::connect(mMmsdManager, SIGNAL(serviceRemoved(const QString&)), SLOT(onMMSDServiceRemoved(const QString&)));
+
+    // workaround: we can't add services here as tp-ofono isn't connected yet
+    QTimer::singleShot(1000, this, SLOT(onCheckMMSServices()));
+}
+
+void oFonoConnection::onCheckMMSServices()
+{
+    Q_FOREACH(QString servicePath, mMmsdManager->services()) {
+        onMMSDServiceAdded(servicePath);
+    }
+}
+
+void oFonoConnection::onMMSDServiceAdded(const QString &path)
+{
+    qDebug() << "oFonoConnection::onMMSServiceAdded" << path;
+    MMSDService *service = new MMSDService(path, this);
+    mMmsdServices[path] = service;
+    QObject::connect(service, SIGNAL(messageAdded(const QString&, const QVariantMap&)), SLOT(onMMSAdded(const QString&, const QVariantMap&)));
+    QObject::connect(service, SIGNAL(messageRemoved(const QString&)), SLOT(onMMSRemoved(const QString&)));
+    Q_FOREACH(MessageStruct message, service->messages()) {
+        addMMStoService(message.path.path(), message.properties, service->path());
+    }
+}
+
+void oFonoConnection::onMMSPropertyChanged(QString property, QVariant value)
+{
+    qDebug() << "oFonoConnection::onMMSPropertyChanged" << property << value;
+    if (property == "Status") {
+        if (value == "sent") {
+            // send delivery report
+        }
+    }
+}
+
+void oFonoConnection::onMMSDServiceRemoved(const QString &path)
+{
+    MMSDService *service = mMmsdServices.take(path);
+    if (!service) {
+        qDebug() << "oFonoConnection::onMMSServiceRemoved failed" << path;
+        return;
+    }
+
+    // remove all messages from this service
+    Q_FOREACH(MMSDMessage *message, mServiceMMSList[service->path()]) {
+        qDebug() << "removing message " <<  message->path() << " from service " << service->path();
+        message->deleteLater();
+        mServiceMMSList[service->path()].removeAll(message);
+    }
+    mServiceMMSList.remove(service->path());
+    service->deleteLater();
+    qDebug() << "oFonoConnection::onMMSServiceRemoved" << path;
+}
+
+void oFonoConnection::addMMStoService(const QString &path, const QVariantMap &properties, const QString &servicePath)
+{
+    qDebug() << "addMMStoService " << path << properties << servicePath;
+    MMSDMessage *msg = new MMSDMessage(path, properties);
+    QObject::connect(msg, SIGNAL(propertyChanged(QString,QVariant)), SLOT(onMMSPropertyChanged(QString,QVariant)));
+    mServiceMMSList[servicePath].append(msg);
+    if (properties["Status"] ==  "received") {
+        const QString normalizedNumber = PhoneNumberUtils::normalizePhoneNumber(properties["Sender"].toString());
+        if (!PhoneNumberUtils::isPhoneNumber(normalizedNumber)) {
+            qDebug() << "Error creating channel for incoming message";
+            return;
+        }
+
+        // check if there is an open channel for this number and use it
+        Q_FOREACH(const QString &phoneNumber, mTextChannels.keys()) {
+            if (PhoneNumberUtils::compareLoosely(normalizedNumber, phoneNumber)) {
+                qDebug() << "existing channel" << mTextChannels[phoneNumber];
+                mTextChannels[phoneNumber]->mmsReceived(path, properties);
+                return;
+            }
+        }
+
+        Tp::DBusError error;
+        bool yours;
+        qDebug() << "new handle" << normalizedNumber;
+        uint handle = newHandle(normalizedNumber);
+        ensureChannel(TP_QT_IFACE_CHANNEL_TYPE_TEXT,Tp::HandleTypeContact, handle, yours, handle, false, &error);
+        if(error.isValid()) {
+            qDebug() << "Error creating channel for incoming message";
+            return;
+        }
+        mTextChannels[normalizedNumber]->mmsReceived(path, properties);
+    }
+}
+
+void oFonoConnection::onMMSAdded(const QString &path, const QVariantMap &properties)
+{
+    qDebug() << "oFonoConnection::onMMSAdded" << path << properties;
+    MMSDService *service = qobject_cast<MMSDService*>(sender());
+    if (!service) {
+        qDebug() << "oFonoConnection::onMMSAdded failed";
+        return;
+    }
+
+    addMMStoService(path, properties, service->path());
+}
+
+void oFonoConnection::onMMSRemoved(const QString &path)
+{
+    qDebug() << "oFonoConnection::onMMSRemoved" << path;
+    MMSDService *service = qobject_cast<MMSDService*>(sender());
+    if (!service) {
+        qDebug() << "oFonoConnection::onMMSRemoved failed";
+        return;
+    }
+
+    // remove this message from the service
+    Q_FOREACH(MMSDMessage *message, mServiceMMSList[service->path()]) {
+        if (message->path() == path) {
+            message->deleteLater();
+            mServiceMMSList[service->path()].removeAll(message);
+            break;
+        }
+    }
 }
 
 oFonoConnection::~oFonoConnection() {
@@ -147,6 +271,10 @@
     mOfonoCallVolume->deleteLater();
     mOfonoNetworkRegistration->deleteLater();
     mRegisterTimer->deleteLater();
+    Q_FOREACH(MMSDService *service, mMmsdServices) {
+        onMMSDServiceRemoved(service->path());
+    }
+   
 }
 
 void oFonoConnection::onTryRegister()
@@ -326,11 +454,25 @@
     }
 
     mTextChannels[newPhoneNumber] = new oFonoTextChannel(this, newPhoneNumber, targetHandle);
+    QObject::connect(mTextChannels[newPhoneNumber], SIGNAL(messageRead(QString)), SLOT(onMessageRead(QString)));
     QObject::connect(mTextChannels[newPhoneNumber], SIGNAL(destroyed()), SLOT(onTextChannelClosed()));
     qDebug() << mTextChannels[newPhoneNumber];
     return mTextChannels[newPhoneNumber]->baseChannel();
 }
 
+void oFonoConnection::onMessageRead(const QString &id)
+{
+    Q_FOREACH(QList<MMSDMessage*> messages, mServiceMMSList.values()) {
+        Q_FOREACH(MMSDMessage* message, messages) {
+            if (message->path() == id) {
+                message->markRead();
+                message->remove();
+                return;
+            }
+        }
+    }
+}
+
 Tp::BaseChannelPtr oFonoConnection::createCallChannel(uint targetHandleType,
                                                uint targetHandle, Tp::DBusError *error)
 {

=== modified file 'connection.h'
--- connection.h	2013-06-01 01:50:34 +0000
+++ connection.h	2013-07-08 15:55:12 +0000
@@ -42,10 +42,14 @@
 #include "ofonotextchannel.h"
 #include "ofonocallchannel.h"
 #include "voicemailiface.h"
+#include "mmsdmanager.h"
+#include "mmsdmessage.h"
+#include "dbustypes.h"
 
 class oFonoConnection;
 class oFonoTextChannel;
 class oFonoCallChannel;
+class MMSDService;
 
 class oFonoConnection : public Tp::BaseConnection
 {
@@ -96,10 +100,16 @@
     void onTextChannelClosed();
     void onCallChannelClosed();
     void onValidityChanged(bool valid);
-
-
+    void onMMSDServiceAdded(const QString&);
+    void onMMSDServiceRemoved(const QString&);
+    void onMMSAdded(const QString &, const QVariantMap&);
+    void onMMSRemoved(const QString &);
+    void onMMSPropertyChanged(QString property, QVariant value);
+    void onCheckMMSServices();
+    void onMessageRead(const QString &id);
 private:
     bool isNetworkRegistered();
+    void addMMStoService(const QString &path, const QVariantMap &properties, const QString &servicePath);
     QMap<uint, QString> mHandles;
 
     QMap<QString, oFonoTextChannel*> mTextChannels;
@@ -116,6 +126,9 @@
     Tp::SimplePresence mSelfPresence;
     Tp::SimplePresence mRequestedSelfPresence;
     QTimer *mRegisterTimer;
+    MMSDManager *mMmsdManager;
+    QMap<QString, MMSDService*> mMmsdServices;
+    QMap<QString, QList<MMSDMessage*> > mServiceMMSList;
 };
 
 #endif

=== added file 'dbustypes.h'
--- dbustypes.h	1970-01-01 00:00:00 +0000
+++ dbustypes.h	2013-07-08 15:55:12 +0000
@@ -0,0 +1,44 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#ifndef DBUSTYPES
+#define DBUSTYPES
+
+struct MessageStruct {
+    QDBusObjectPath path;
+    QVariantMap properties;
+};
+
+struct AttachmentStruct {
+    QString id;
+    QString contentType;
+    QString filePath;
+    quint64 offset;
+    quint64 length;
+};
+
+typedef QList<AttachmentStruct> AttachmentList;
+Q_DECLARE_METATYPE(AttachmentStruct)
+Q_DECLARE_METATYPE(AttachmentList)
+
+
+typedef QList<MessageStruct> MessageList;
+Q_DECLARE_METATYPE(MessageStruct)
+Q_DECLARE_METATYPE(MessageList)
+
+#endif

=== added file 'mmsdmanager.cpp'
--- mmsdmanager.cpp	1970-01-01 00:00:00 +0000
+++ mmsdmanager.cpp	2013-07-08 15:55:12 +0000
@@ -0,0 +1,99 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#include <QtDBus>
+#include <QObject>
+
+#include "mmsdmanager.h"
+
+struct ServiceStruct {
+    QDBusObjectPath path;
+    QVariantMap properties;
+};
+
+typedef QList<ServiceStruct> ServiceList;
+Q_DECLARE_METATYPE(ServiceStruct)
+Q_DECLARE_METATYPE(ServiceList)
+
+QDBusArgument &operator<<(QDBusArgument &argument, const ServiceStruct &service)
+{
+    argument.beginStructure();
+    argument << service.path << service.properties;
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, ServiceStruct &service)
+{
+    argument.beginStructure();
+    argument >> service.path >> service.properties;
+    argument.endStructure();
+    return argument;
+}
+
+MMSDManager::MMSDManager(QObject *parent)
+    : QObject(parent)
+{
+    QDBusReply<ServiceList> reply;
+    ServiceList services;
+
+    QDBusMessage request;
+
+    qDBusRegisterMetaType<ServiceStruct>();
+    qDBusRegisterMetaType<ServiceList>();
+
+    request = QDBusMessage::createMethodCall("org.ofono.mms",
+                                             "/org/ofono/mms", "org.ofono.mms.Manager",
+                                             "GetServices");
+    reply = QDBusConnection::sessionBus().call(request);
+
+    services = reply;
+    Q_FOREACH(ServiceStruct service, services) {
+        m_services << service.path.path();
+    }
+
+    QDBusConnection::sessionBus().connect("org.ofono.mms","/org/ofono/mms","org.ofono.mms.Manager",
+                                          "ServiceAdded", this, 
+                                          SLOT(onServiceAdded(const QDBusObjectPath&, const QVariantMap&)));
+    QDBusConnection::sessionBus().connect("org.ofono.mms","/org/ofono/mms","org.ofono.mms.Manager",
+                                          "ServiceRemoved", this, 
+                                          SLOT(onServiceRemoved(const QDBusObjectPath&)));
+}
+
+MMSDManager::~MMSDManager()
+{
+}
+
+QStringList MMSDManager::services() const
+{
+    return m_services;
+}
+
+void MMSDManager::onServiceAdded(const QDBusObjectPath& path, const QVariantMap& map)
+{
+    qDebug() << "service added" << path.path() << map;
+    m_services << path.path();
+    Q_EMIT serviceAdded(path.path());
+}
+
+void MMSDManager::onServiceRemoved(const QDBusObjectPath& path)
+{
+    qDebug() << "service removed" << path.path();
+    m_services.removeAll(path.path());
+    Q_EMIT serviceRemoved(path.path());
+}

=== added file 'mmsdmanager.h'
--- mmsdmanager.h	1970-01-01 00:00:00 +0000
+++ mmsdmanager.h	2013-07-08 15:55:12 +0000
@@ -0,0 +1,48 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#ifndef MMSDMANAGER_H
+#define MMSDMANAGER_H
+
+#include <QObject>
+#include <QVariantMap>
+#include <QDBusObjectPath>
+#include <QStringList>
+
+class MMSDManager : public QObject
+{
+    Q_OBJECT
+public:
+    MMSDManager(QObject *parent=0);
+    ~MMSDManager();
+
+    QStringList services() const;
+
+Q_SIGNALS:
+    void serviceAdded(const QString &servicePath);
+    void serviceRemoved(const QString &servicePath);
+
+private Q_SLOTS:
+    void onServiceAdded(const QDBusObjectPath &path, const QVariantMap &properties);
+    void onServiceRemoved(const QDBusObjectPath &path);
+
+private:
+    QStringList m_services;
+};
+
+#endif

=== added file 'mmsdmessage.cpp'
--- mmsdmessage.cpp	1970-01-01 00:00:00 +0000
+++ mmsdmessage.cpp	2013-07-08 15:55:12 +0000
@@ -0,0 +1,71 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#include <QtDBus>
+#include <QObject>
+
+#include "mmsdmessage.h"
+
+MMSDMessage::MMSDMessage(QString objectPath, QVariantMap properties, QObject *parent)
+    : QObject(parent), 
+      m_messagePath(objectPath),
+      m_properties(properties)
+{
+    QDBusConnection::sessionBus().connect("org.ofono.mms", m_messagePath, "org.ofono.mms.Message",
+                                          "PropertyChanged", this,
+                                          SLOT(onPropertyChanged(const QString&, const QVariant&)));
+}
+
+MMSDMessage::~MMSDMessage()
+{
+}
+
+QString MMSDMessage::path() const
+{
+    return m_messagePath;
+}
+
+QVariantMap MMSDMessage::properties() const
+{
+    return m_properties;
+}
+
+void MMSDMessage::onPropertyChanged(const QString &property, const QVariant &value)
+{
+    qDebug() << "property changed" << property << value;
+    m_properties[property] = value;
+    Q_EMIT propertyChanged(property, value);
+}
+
+void MMSDMessage::markRead() const
+{
+    QDBusMessage request;
+    request = QDBusMessage::createMethodCall("org.ofono.mms",
+                                   m_messagePath, "org.ofono.mms.Message",
+                                   "MarkRead");
+    QDBusConnection::sessionBus().call(request);
+}
+
+void MMSDMessage::remove() const
+{
+    QDBusMessage request;
+    request = QDBusMessage::createMethodCall("org.ofono.mms",
+                                   m_messagePath, "org.ofono.mms.Message",
+                                   "Delete");
+    QDBusConnection::sessionBus().call(request);
+}

=== added file 'mmsdmessage.h'
--- mmsdmessage.h	1970-01-01 00:00:00 +0000
+++ mmsdmessage.h	2013-07-08 15:55:12 +0000
@@ -0,0 +1,51 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#ifndef MMSDMESSAGE_H
+#define MMSDMESSAGE_H
+
+#include <QObject>
+#include <QVariantMap>
+#include <QDBusObjectPath>
+#include <QStringList>
+
+class MMSDMessage : public QObject
+{
+    Q_OBJECT
+public:
+    MMSDMessage(QString objectPath, QVariantMap properties, QObject *parent=0);
+    ~MMSDMessage();
+
+    QVariantMap properties() const;
+    QString path() const;
+    void markRead() const;
+    // it should be called delete, but it is a reserved keyword in c++
+    void remove() const;
+
+Q_SIGNALS:
+    void propertyChanged(const QString&, const QVariant&);
+
+private Q_SLOTS:
+    void onPropertyChanged(const QString&, const QVariant&);
+
+private:
+    QVariantMap m_properties;
+    QString m_messagePath;
+};
+
+#endif

=== added file 'mmsdservice.cpp'
--- mmsdservice.cpp	1970-01-01 00:00:00 +0000
+++ mmsdservice.cpp	2013-07-08 15:55:12 +0000
@@ -0,0 +1,120 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#include <QtDBus>
+#include <QObject>
+
+#include "dbustypes.h"
+#include "mmsdservice.h"
+
+QDBusArgument &operator<<(QDBusArgument &argument, const MessageStruct &message)
+{
+    argument.beginStructure();
+    argument << message.path << message.properties;
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, MessageStruct &message)
+{
+    argument.beginStructure();
+    argument >> message.path >> message.properties;
+    argument.endStructure();
+    return argument;
+}
+
+MMSDService::MMSDService(QString objectPath, oFonoConnection* connection, QObject *parent)
+    : QObject(parent), 
+      m_servicePath(objectPath)
+{
+    QDBusReply<MessageList> replyMessages;
+    QDBusReply<QVariantMap> replyProperties;
+
+    QDBusMessage request;
+
+    qDBusRegisterMetaType<MessageStruct>();
+    qDBusRegisterMetaType<MessageList>();
+
+    request = QDBusMessage::createMethodCall("org.ofono.mms",
+                                             m_servicePath, "org.ofono.mms.Service",
+                                             "GetProperties");
+    replyProperties = QDBusConnection::sessionBus().call(request);
+    m_properties = replyProperties;
+
+    request = QDBusMessage::createMethodCall("org.ofono.mms",
+                                             m_servicePath, "org.ofono.mms.Service",
+                                             "GetMessages");
+    replyMessages = QDBusConnection::sessionBus().call(request);
+
+    m_messages = replyMessages;
+
+    QDBusConnection::sessionBus().connect("org.ofono.mms", m_servicePath, "org.ofono.mms.Service",
+                                          "MessageAdded", this,
+                                          SLOT(onMessageAdded(const QDBusObjectPath&, const QVariantMap&)));
+    QDBusConnection::sessionBus().connect("org.ofono.mms", m_servicePath, "org.ofono.mms.Service",
+                                          "MessageRemoved", this,
+                                          SLOT(onMessageRemoved(const QDBusObjectPath&)));
+}
+
+MMSDService::~MMSDService()
+{
+}
+
+QString MMSDService::path() const
+{
+    return m_servicePath;
+}
+
+QVariantMap MMSDService::properties() const
+{
+    return m_properties;
+}
+
+MessageList MMSDService::messages() const
+{
+    return m_messages;
+}
+
+void MMSDService::onMessageAdded(const QDBusObjectPath &path, const QVariantMap &properties)
+{
+    qDebug() << "message added" << path.path() << properties;
+    Q_EMIT messageAdded(path.path(), properties);
+}
+
+void MMSDService::onMessageRemoved(const QDBusObjectPath& path)
+{
+    qDebug() << "message removed" << path.path();
+    Q_EMIT messageRemoved(path.path());
+}
+
+QDBusObjectPath MMSDService::sendMessage(QStringList recipients, QString smil, AttachmentList attachments)
+{
+    QDBusMessage request;
+    QList<QVariant> arguments;
+    QDBusReply<QDBusObjectPath> reply;
+    arguments.append(recipients);
+    arguments.append(smil);
+    arguments.append(QVariant::fromValue(attachments));
+    request = QDBusMessage::createMethodCall("org.ofono.mms",
+                                             m_servicePath, "org.ofono.mms.Service",
+                                             "SendMessage");
+    request.setArguments(arguments);
+    reply = QDBusConnection::sessionBus().call(request);
+
+    return reply;
+}

=== added file 'mmsdservice.h'
--- mmsdservice.h	1970-01-01 00:00:00 +0000
+++ mmsdservice.h	2013-07-08 15:55:12 +0000
@@ -0,0 +1,58 @@
+/**
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU Lesser General Public License version 3, as published by
+ * the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
+ * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
+ */
+
+#ifndef MMSDSERVICE_H
+#define MMSDSERVICE_H
+
+#include <QObject>
+#include <QVariantMap>
+#include <QDBusObjectPath>
+#include <QStringList>
+
+#include "dbustypes.h"
+#include "connection.h"
+#include "mmsdmessage.h"
+
+class MMSDService : public QObject
+{
+    Q_OBJECT
+public:
+    MMSDService(QString objectPath, oFonoConnection *connection, QObject *parent=0);
+    ~MMSDService();
+
+    QVariantMap properties() const;
+    MessageList messages() const;
+    QString path() const;
+
+    QDBusObjectPath sendMessage(QStringList recipients, QString smil, AttachmentList attachments);
+
+Q_SIGNALS:
+    void messageAdded(const QString &messagePath, const QVariantMap &properties);
+    void messageRemoved(const QString &messagePath);
+
+private Q_SLOTS:
+    void onMessageAdded(const QDBusObjectPath &path, const QVariantMap &properties);
+    void onMessageRemoved(const QDBusObjectPath &path);
+
+private:
+    QVariantMap m_properties;
+    QString m_servicePath;
+    MessageList m_messages;
+};
+
+#endif

=== modified file 'ofonotextchannel.cpp'
--- ofonotextchannel.cpp	2013-06-01 01:50:34 +0000
+++ ofonotextchannel.cpp	2013-07-08 15:55:12 +0000
@@ -22,6 +22,22 @@
 // telepathy-ofono
 #include "ofonotextchannel.h"
 
+QDBusArgument &operator<<(QDBusArgument &argument, const AttachmentStruct &attachment)
+{
+    argument.beginStructure();
+    argument << attachment.id << attachment.contentType << attachment.filePath << attachment.offset << attachment.length;
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, AttachmentStruct &attachment)
+{
+    argument.beginStructure();
+    argument >> attachment.id >> attachment.contentType >> attachment.filePath >> attachment.offset >> attachment.length;
+    argument.endStructure();
+    return argument;
+}
+
 oFonoTextChannel::oFonoTextChannel(oFonoConnection *conn, QString phoneNumber, uint targetHandle, QObject *parent):
     QObject(parent),
     mConnection(conn),
@@ -29,6 +45,9 @@
     mTargetHandle(targetHandle),
     mMessageCounter(1)
 {
+    qDBusRegisterMetaType<AttachmentStruct>();
+    qDBusRegisterMetaType<AttachmentList>();
+
     Tp::BaseChannelPtr baseChannel = Tp::BaseChannel::create(mConnection,
                                                              TP_QT_IFACE_CHANNEL_TYPE_TEXT,
                                                              targetHandle,
@@ -54,6 +73,7 @@
     baseChannel->plugInterface(Tp::AbstractChannelInterfacePtr::dynamicCast(mMessagesIface));
     mBaseChannel = baseChannel;
     mTextChannel = Tp::BaseChannelTextTypePtr::dynamicCast(mBaseChannel->interface(TP_QT_IFACE_CHANNEL_TYPE_TEXT));
+    mTextChannel->setMessageAcknowledgedCallback(Tp::memFun(this,&oFonoTextChannel::messageAcknowledged));
     QObject::connect(mBaseChannel.data(), SIGNAL(closed()), this, SLOT(deleteLater()));
 }
 
@@ -66,6 +86,11 @@
     return mBaseChannel;
 }
 
+void oFonoTextChannel::messageAcknowledged(const QString &id)
+{
+    Q_EMIT messageRead(id);
+}
+
 QString oFonoTextChannel::sendMessage(const Tp::MessagePartList& message, uint flags, Tp::DBusError* error)
 {
     bool success = true;
@@ -133,3 +158,40 @@
 
     mTextChannel->addReceivedMessage(partList);
 }
+
+void oFonoTextChannel::mmsReceived(const QString &id, const QVariantMap &properties)
+{
+    Tp::MessagePartList message;
+
+    Tp::MessagePart header;
+    header["message-token"] = QDBusVariant(id);
+    header["message-sender"] = QDBusVariant(mTargetHandle);
+    header["message-received"] = QDBusVariant(QDateTime::fromString(properties["Date"].toString(), Qt::ISODate).toTime_t());
+    header["message-type"] = QDBusVariant(Tp::DeliveryStatusDelivered);
+    message << header;
+    if (!properties["Subject"].toString().isEmpty())
+    {
+        Tp::MessagePart part;
+        part["content-type"] =  QDBusVariant("text/plain");
+        part["content"] = QDBusVariant(properties["Subject"].toString());
+        message << part;
+    }
+    AttachmentList a = qdbus_cast<AttachmentList>(properties["Attachments"]);
+    Q_FOREACH(AttachmentStruct b, a) {
+        QFile a(b.filePath);
+        if (!a.open(QIODevice::ReadOnly)) {
+            qDebug() << "fail to load attachment";
+        }
+        a.seek(b.offset);
+        QByteArray fileData = a.read(b.length);
+        Tp::MessagePart part;
+        part["content-type"] =  QDBusVariant(b.contentType);
+        part["identifier"] = QDBusVariant(b.id);
+        part["content"] = QDBusVariant(fileData);
+        part["size"] = QDBusVariant(b.length);
+
+        message << part;
+    }
+
+    mTextChannel->addReceivedMessage(message);
+}

=== modified file 'ofonotextchannel.h'
--- ofonotextchannel.h	2013-06-01 01:50:34 +0000
+++ ofonotextchannel.h	2013-07-08 15:55:12 +0000
@@ -24,6 +24,7 @@
 #include <TelepathyQt/Constants>
 #include <TelepathyQt/BaseChannel>
 #include <TelepathyQt/Types>
+#include <TelepathyQt/DBusError>
 
 #include "connection.h"
 
@@ -37,10 +38,15 @@
     QString sendMessage(const Tp::MessagePartList& message, uint flags, Tp::DBusError* error);
     void messageReceived(const QString & message, const QVariantMap &info);
     Tp::BaseChannelPtr baseChannel();
+    void messageAcknowledged(const QString &id);
+    void mmsReceived(const QString &id, const QVariantMap &properties);
 
 private Q_SLOTS:
     void onOfonoMessageStateChanged(QString status);
 
+Q_SIGNALS:
+    void messageRead(const QString &id);
+
 private:
     ~oFonoTextChannel();
     Tp::BaseChannelPtr mBaseChannel;

